@@ -9,6 +9,7 @@ static char rcsid[] = "$Id: host2ip.c,v 1.2 1995/07/13 18:03:31 hgs Exp $";
 
 /*
 * Return IP address given host name 'host'.
+* If 'host' is "", set to INADDR_ANY.
 */
 struct in_addr host2ip(char *host)
 {
@@ -16,7 +17,7 @@ struct in_addr host2ip(char *host)
   register struct hostent *hep;
 
   /* Check whether this is a dotted decimal. */
-  if (!host) {
+  if (!host || *host == '\0') {
     in.s_addr = INADDR_ANY;
   }
   else if ((in.s_addr = inet_addr(host)) != -1) {
