@@ -181,7 +181,7 @@ struct timeval *timer_set(struct timeval *interval,
 * so the select() will just block.  Otherwise, the supplied timeval struct is
 * filled with the timeout interval until the next timer expires.
 
-* Note:  This routine may be called recusively if the timer event handling
+* Note:  This routine may be called recursively if the timer event handling
 * routine leads to another select() call!  Therefore, we just take one timer
 * at a time, and don't use static variables.
 */
@@ -220,3 +220,12 @@ struct timeval *timer_get(struct timeval *timeout)
     }
   } /* loop to see if another timer expired */
 } /* timer_get */
+
+
+/*
+* Return 1 if the timer queue is not empty.
+*/
+int timer_pending(void)
+{
+  return timerQ != 0; 
+} /* timer_pending */
