@@ -49,7 +49,7 @@ int RD_read(FILE *in, RD_buffer_t *b)
   /* read packet header from file */
   if (fread((char *)b->byte, sizeof(b->p.hdr), 1, in) == 0) {
     /* we are done */
-    exit(0);
+    return 0;
   }
 
   /* convert to host byte order */
