@@ -136,9 +136,11 @@ static int open_network(char *host, int data, int sock[], struct
     }
     if (sock[i] > nfds) nfds = sock[i];
 
-    if (setsockopt(sock[i], SOL_SOCKET, SO_REUSEADDR, (char *) &one,
+    if (IN_CLASSD(ntohl(mreq.imr_multiaddr.s_addr))) {
+      if (setsockopt(sock[i], SOL_SOCKET, SO_REUSEADDR, (char *) &one,
                    sizeof(one)) == -1)
-      perror("setsockopt: reuseaddr");
+        perror("setsockopt: reuseaddr");
+    }
 
 #ifdef SO_REUSEPORT /* BSD 4.4 */
     if (setsockopt(sock[i], SOL_SOCKET, SO_REUSEPORT, (char *) &one,
@@ -685,6 +687,8 @@ int main(int argc, char *argv[])
 
   /* signal handler */
   signal(SIGINT, done);
+  signal(SIGTERM, done);
+  signal(SIGHUP, done);
 
   /* main loop */
   while (1) {
@@ -726,6 +730,7 @@ int main(int argc, char *argv[])
     }
     else {
       len = RD_read(in, &packet);
+      if (len == 0) exit(0);
       now.tv_sec = packet.p.hdr.offset / 1000.;
       now.tv_usec = (packet.p.hdr.offset % 1000) * 1000;
       dnow = tdbl(&now);
