@@ -4,13 +4,15 @@
 * -v         verbose
 * -T         use absolute time rather than RTP timestamps
 * -f         file to read
+* -b         begin time
+* -e         end time
 * -p [file]  profile of RTP PT to frequency mappings
 *
 * Program reads ahead by READAHEAD packets to compensate for reordering.
 * Currently does not correct SR/RR absolute (NTP) timestamps,
 * but should. Receiver reports are fairly meaningless.
 *
-* (c) 1994-1996 Henning Schulzrinne
+* (c) 1994-1998 Henning Schulzrinne
 */
 
 #include <stdio.h>
@@ -35,11 +37,13 @@
 #define READAHEAD 16 /* must be power of 2 */
 
 static char rcsid[] = "$Id: rtpplay.c,v 1.2 1995/08/30 11:19:01 hgs Exp $";
-static int verbose = 0;     /* be chatty about packets sent */
-static int wallclock = 0;   /* use wallclock time rather than timestamps */
-static FILE *in = stdin;    /* input file */
-static int sock[2];         /* output sockets */
-static int first = -1;      /* offset of first packet */
+static int verbose = 0;        /* be chatty about packets sent */
+static int wallclock = 0;      /* use wallclock time rather than timestamps */
+static u_int32 begin = 0;      /* time of first packet to send */
+static u_int32 end = UINT_MAX; /* when to stop sending */ 
+static FILE *in = stdin;       /* input file */
+static int sock[2];            /* output sockets */
+static int first = -1;         /* time offset of first packet */
 static RD_buffer_t buffer[READAHEAD];
 
 static double period[128] = {  /* ms per timestamp difference */
@@ -84,7 +88,7 @@ static double period[128] = {  /* ms per timestamp difference */
 static void usage(char *argv0)
 {
   fprintf(stderr,
-"Usage: %s [-v] [-T] [-p profile] [-f file] destination/port[/ttl]\n",
+"Usage: %s [-v] [-T] [-p profile] [-f file] [-b begin time] [-e end time] destination/port[/ttl]\n",
   argv0);
   exit(1);
 } /* usage */
@@ -93,7 +97,7 @@ static void usage(char *argv0)
 static double tdbl(struct timeval *a)
 {
   return a->tv_sec + a->tv_usec/1e6;
-}
+} /* tdbl */
 
 
 /*
@@ -106,6 +110,7 @@ static void play_transmit(int b)
         buffer[b].p.data, buffer[b].p.hdr.length) < 0) {
       perror("write");
     } 
+
     buffer[b].p.hdr.length = 0;
   }
 } /* play_transmit */
@@ -140,31 +145,55 @@ static Notify_value play_handler(Notify_client client)
   /* playback scheduled packet */
   play_transmit(b);
 
+  /* If we are done, skip rest. */
+  if (end == 0) return NOTIFY_DONE;
+
   if (verbose > 0 && b >= 0) {
-    printf("%1.3f %s(%3d;%3d) t=%6lu",
+    printf("! %1.3f %s(%3d;%3d) t=%6lu",
       tdbl(&now), buffer[b].p.hdr.plen ? "RTP " : "RTCP",
       buffer[b].p.hdr.length, buffer[b].p.hdr.plen,
       (unsigned long)buffer[b].p.hdr.offset);
 
     if (buffer[b].p.hdr.plen) {
       r = (rtp_hdr_t *)buffer[b].p.data;
-      printf(" ssrc=%8lx %cts=%9ld seq=%5d",
+      printf(" ssrc=%8lx %cts=%9lu seq=%5u",
         ntohl(r->ssrc), r->m ? '*' : ' ',
         ntohl(r->ts), ntohs(r->seq));
     }
     printf("\n");
   }
 
-  /* find available buffer */
+  /* Find available buffer. */
   for (rp = 0; rp < READAHEAD; rp++) {
     if (!buffer[rp].p.hdr.length) break;
   }
 
-  /* get next packet */
-  RD_read(in, &buffer[rp]);
+  /* Get next packet; try again if we haven't reached the begin time. */
+  do {
+    if (RD_read(in, &buffer[rp]) == 0) return NOTIFY_DONE;
+  } while (buffer[rp].p.hdr.offset < begin);
+
+  /* 
+   * If new packet is after end of alloted time, don't insert into list
+   * and set 'end' to zero to avoid reading any more packets from
+   * file.
+   */
+  if (buffer[rp].p.hdr.offset > end) {
+    buffer[rp].p.hdr.length = 0; /* erase again */
+    end = 0;
+    return NOTIFY_DONE;
+  }
 
   r = (rtp_hdr_t *)buffer[rp].p.data;
 
+  /* Remember wallclock and recording time of first valid packet. */
+  if (first < 0) {
+    start = now;
+    first = buffer[rp].p.hdr.offset;
+  }
+  buffer[rp].p.hdr.offset -= first;
+
+  /* RTP played according to timestamp. */
   if (buffer[rp].p.hdr.plen && r->version == 2 && !wallclock) {
     ENTRY item;
 
@@ -179,7 +208,7 @@ static Notify_value play_handler(Notify_client client)
     item.data = 0;
     e = hsearch(item, FIND);
 
-    /* if found, compute playout instant */
+    /* If found in list of sources, compute playout instant. */
     if (e) {
       double d;
 
@@ -187,23 +216,23 @@ static Notify_value play_handler(Notify_client client)
       d = period[pt] * (int)(ts - t->ts);
       next.tv_sec  = t->rt.tv_sec  + (int)d;
       next.tv_usec = t->rt.tv_usec + (d - (int)d) * 1000000;
-      printf("%1.3f rp=%d b=%d d=%f\n", tdbl(&next), rp, b, d);
-    } else { /* if none, insert and play now */
+      if (verbose)
+        printf(". %1.3f t=%6lu ts=%lu,%lu rp=%2d b=%d d=%f\n", tdbl(&next), 
+        (unsigned long)buffer[rp].p.hdr.offset, 
+        (unsigned long)ts, (unsigned long)t->ts,
+        rp, b, d);
+    } else { /* If not on source list, insert and play based on wallclock. */
       item.key  = malloc(strlen(ssrc)+1);
       strcpy(item.key, ssrc);
       item.data = (void *)t = (struct rt_ts *)malloc(sizeof(struct rt_ts));
-      next = now;
+      next.tv_sec  = start.tv_sec  + buffer[rp].p.hdr.offset/1000;
+      next.tv_usec = start.tv_usec + (buffer[rp].p.hdr.offset%1000) * 1000;
       e = hsearch(item, ENTER);
     }
   }
-  /* RTCP or vat or playing back by wallclock */
+  /* RTCP or vat or playing back by wallclock. */
   else {
-    if (first < 0) {
-      start = now;
-      first = buffer[rp].p.hdr.offset;
-    }
     /* compute next playout time */
-    buffer[rp].p.hdr.offset -= first;
     next.tv_sec  = start.tv_sec  + buffer[rp].p.hdr.offset/1000;
     next.tv_usec = start.tv_usec + (buffer[rp].p.hdr.offset%1000) * 1000;
     ssrc[0] = '\0';
@@ -213,13 +242,13 @@ static Notify_value play_handler(Notify_client client)
     next.tv_usec -= 1000000;
     next.tv_sec  += 1;
   }
-  /* save correct value in record (for timestamp-based playback) */
+
+  /* Save correct value in record (for timestamp-based playback). */
   if (t) {
     t->rt = next;
     t->ts = ts; 
   }
 
-
   timer_set(&next, play_handler, (Notify_client)rp, 0);
   return NOTIFY_DONE;
 } /* play_handler */
@@ -259,8 +288,14 @@ int main(int argc, char *argv[])
   extern int hpt(char *h, struct sockaddr *sa, unsigned char *ttl);
 
   /* parse command line arguments */
-  while ((c = getopt(argc, argv, "f:p:Tv")) != EOF) {
+  while ((c = getopt(argc, argv, "b:e:f:p:Tv")) != EOF) {
     switch(c) {
+    case 'b':
+      begin = atof(optarg) * 1000;
+      break;
+    case 'e':
+      end = atof(optarg) * 1000;
+      break;
     case 'f':
       if (!(in = fopen(optarg, "r"))) {
         perror(optarg);
